
use AdventureWorks
go

-- Clustered Index Scan
SELECT SalesOrderID 
    , COUNT(*) AS [ItemCount]
    , SUM(LineTotal) AS [LineItemSum]
FROM Sales.SalesOrderDetail 
WHERE UnitPriceDiscount > 0
GROUP BY GROUPING SETS ((SalesOrderID), ());

-- Look at the distribution
select UnitPriceDiscount, COUNT(*) as [RowCount], 
	(COUNT(*) * 100.00)/(select COUNT(*) from Sales.SalesOrderDetail) as [Percent]
from Sales.SalesOrderDetail
group by UnitPriceDiscount;

-- Ordinary Index
CREATE NONCLUSTERED INDEX IX_Sales_SalesOrderDetail_UnitPriceDiscount
    ON Sales.SalesOrderDetail(UnitPriceDiscount)
    Include (SalesOrderID, LineTotal);
    
SET STATISTICS IO ON; 
DBCC DROPCLEANBUFFERS;     

SELECT SalesOrderID 
    , COUNT(*) AS [ItemCount]
    , SUM(LineTotal) AS [LineItemSum]
FROM Sales.SalesOrderDetail WITH 
    (INDEX(PK_SalesOrderDetail_SalesOrderID_SalesOrderDetailID))
WHERE UnitPriceDiscount > 0
GROUP BY SalesOrderID; 

DBCC DROPCLEANBUFFERS; 
   
SELECT SalesOrderID 
    , COUNT(*) AS [ItemCount]
    , SUM(LineTotal) AS [LineItemSum]
FROM Sales.SalesOrderDetail 
WHERE UnitPriceDiscount > 0
GROUP BY SalesOrderID; 

SET STATISTICS IO OFF;

--Create a filtered index and explore what happens:
CREATE NONCLUSTERED INDEX FIX_Sales_SalesOrderDetail_UnitPriceDiscount_Filtered 
    ON Sales.SalesOrderDetail(SalesOrderID)
    Include (LineTotal)
    WHERE UnitPriceDiscount > 0;
    
--Numver of pages consumed by each index:
SELECT i.name, ddips.index_depth, ddips.index_level 
    , ddips.page_count, ddips.record_count 
FROM sys.indexes AS i 
Join sys.dm_db_index_physical_stats(DB_ID(),  
    OBJECT_ID(N'Sales.SalesOrderDetail'), Null, Null, N'Detailed') AS ddips 
    ON i.OBJECT_ID = ddips.OBJECT_ID
    And i.index_id = ddips.index_id 
WHERE i.name In ('IX_Sales_SalesOrderDetail_UnitPriceDiscount'
    , 'FIX_Sales_SalesOrderDetail_UnitPriceDiscount_Filtered'
    , 'PK_SalesOrderDetail_SalesOrderID_SalesOrderDetailID')
    AND ddips.index_level = 0;

SET STATISTICS IO ON; 
   
DBCC DROPCLEANBUFFERS; 
   
SELECT SalesOrderID 
    , COUNT(*) AS [ItemCount]
    , SUM(LineTotal) AS [LineItemSum]
FROM Sales.SalesOrderDetail WITH (INDEX(IX_Sales_SalesOrderDetail_UnitPriceDiscount))
WHERE UnitPriceDiscount > 0 
GROUP BY SalesOrderID; 
  
DBCC DROPCLEANBUFFERS; 
   
SELECT SalesOrderID 
    , COUNT(*) AS [ItemCount]
    , SUM(LineTotal) AS [LineItemSum]
FROM Sales.SalesOrderDetail 
WHERE UnitPriceDiscount > 0 
GROUP BY SalesOrderID; 
   
SET STATISTICS IO OFF;

select * from sales.salesorderheader

-- This join should use it too
SELECT D.SalesOrderID 
    , D.SalesOrderID / H.SubTotal * 100 AS [DiscountedPercent]
    , COUNT(*) AS [ItemCount]
    , SUM(LineTotal) AS [LineItemSum]
FROM Sales.SalesOrderDetail D 
JOIN Sales.SalesOrderHeader H
  ON H.SalesOrderID = D.SalesOrderID
WHERE UnitPriceDiscount > 0 
GROUP BY D.SalesOrderID, H.SubTotal 

-- This won't use index, unless you add UnitPriceDiscount (not covering)
SELECT SalesOrderID 
    , UnitPriceDiscount 
    , COUNT(*) AS [ItemCount]
    , SUM(LineTotal) AS [LineItemSum]
FROM Sales.SalesOrderDetail 
WHERE UnitPriceDiscount > 0 
GROUP BY SalesOrderID 
    , UnitPriceDiscount;
        

-- Look at metadata
SELECT name, has_filter, filter_definition 
FROM sys.indexes  
WHERE name In ('IX_Sales_SalesOrderDetail_UnitPriceDiscount'
    , 'FIX_Sales_SalesOrderDetail_UnitPriceDiscount_Filtered'
    , 'PK_SalesOrderDetail_SalesOrderID_SalesOrderDetailID');

-- see statistics buckets
DBCC SHOW_STATISTICS ('Sales.SalesOrderDetail', IX_Sales_SalesOrderDetail_UnitPriceDiscount)
DBCC SHOW_STATISTICS ('Sales.SalesOrderDetail', FIX_Sales_SalesOrderDetail_UnitPriceDiscount_Filtered)

-- reset
DROP INDEX FIX_Sales_SalesOrderDetail_UnitPriceDiscount_Filtered ON Sales.SalesOrderDetail
DROP INDEX IX_Sales_SalesOrderDetail_UnitPriceDiscount ON Sales.SalesOrderDetail