-- The following simplified TSQL example shows the problem and the solution.
-- NOTE: the fact that Stored Procedures were used is important, 
-- because they inherit the SET statement values they were created with, 
-- and NOT the values they are executed with. 

USE master
GO
CREATE DATABASE FI_Test
GO
USE FI_Test
GO
SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO

CREATE TABLE [FactSalesQuota](
	[SalesQuotaKey] [int] IDENTITY(1,1) NOT NULL,
	[EmployeeKey] [int] NOT NULL,
	[DateKey] [int] NOT NULL,
	[CalendarYear] [smallint] NOT NULL,
	[CalendarQuarter] [tinyint] NOT NULL,
	[SalesAmountQuota] [money] NOT NULL,
 CONSTRAINT [PK_FactSalesQuota_SalesQuotaKey] PRIMARY KEY CLUSTERED 
([SalesQuotaKey] ASC) ON [PRIMARY]
) ON [PRIMARY]
GO

CREATE NONCLUSTERED INDEX FI_FactSalesQuota 
ON FactSalesQuota(Employeekey,CalendarQuarter) 
WHERE CalendarYear = 2009 -- <---- This makes it a Filtered index
GO

INSERT FactSalesQuota values(53,20090101,2009,4, 37000.00)
GO
SELECT * FROM FactSalesQuota
GO

CREATE PROCEDURE UPDATE_FactSalesQuota 
 @EmployeeKey int,
 @CalendarYear smallint,
 @CalendarQuarter tinyint, 
 @SalesAmountQuota money 
AS 
UPDATE FactSalesQuota 
 SET SalesAmountQuota =  @SalesAmountQuota
     
 WHERE	EmployeeKey = @EmployeeKey and
		CalendarYear = @CalendarYear and
		CalendarQuarter = @CalendarQuarter  
GO

EXECUTE UPDATE_FactSalesQuota 53,2009,4,50000.00
GO
SELECT * FROM FactSalesQuota
GO

--- script to update the sp
use FI_Test
GO
sp_rename UPDATE_FactSalesQuota , UPDATE_FactSalesQuota_V1
GO
SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER OFF -- <------ HERE is what caused the error
--SET QUOTED_IDENTIFIER ON -- <------ it will work if this is set correctly
GO

CREATE PROCEDURE UPDATE_FactSalesQuota 
 @EmployeeKey int,
 @CalendarYear smallint,
 @CalendarQuarter tinyint, 
 @SalesAmountQuota money 
AS 
UPDATE FactSalesQuota 
 SET	SalesAmountQuota =  @SalesAmountQuota,
        DateKey = cast((CONVERT (char(8) ,getdate(), 112)) as int)
 WHERE	EmployeeKey = @EmployeeKey and
		CalendarYear = @CalendarYear and
		CalendarQuarter = @CalendarQuarter  
GO

--this fails
EXECUTE UPDATE_FactSalesQuota 53,2009,4,52000.00
GO
SELECT * FROM FactSalesQuota
GO

DROP DATABASE FI_Test
GO
